--these variables are meant to be easily configurable, if you would like
--note: speed is measured in cm/s

local ROUND_TO_NEAREST = 0 --eg. a value of 100 rounds to the nearest 100; a value of zero does not round at all
local AVERAGE_INTERVAL_TIME = 0 --speed averages are updated after this time period (seconds)
local DISPLAY_DECIMAL_PLACES = 2 --digits after the decimal point "." to be displayed
local LEADING_ZEROES = 3 --digits before the decimal point; empty spaces will be substituted with "0". example: "SPEED: 1" --> "SPEED: 00001", with LEADING_ZEROES = 5

local SPEED_LABEL = "SPEED:"
local FORMAT_STRING = "" --this is placed after the speed number

local BOX_W = 128 --width of box
local BOX_H = 32 --height of box

local BOX_X = 0 --horizontal position
local BOX_Y = 0 --vertical position

local FONT_SIZE = 24 --size of the spedometer text
local TEXT_COLOR = "ffffff" --hexadecimal representation of the spedometer text color

local TEXT_H_MARGIN = 8 --horizontal space between the edge of the spedometer box to the text


--beware of changing things below this line

if DISPLAY_DECIMAL_PLACES > 0 then 
	FORMAT_STRING = "%0" .. tostring(DISPLAY_DECIMAL_PLACES + LEADING_ZEROES + 1) .. "." .. tostring(DISPLAY_DECIMAL_PLACES) .. "f" .. FORMAT_STRING
else
	FORMAT_STRING = "%0" .. tostring(LEADING_ZEROES) .. "i" .. FORMAT_STRING
end
local mvec_copy = mvector3.copy
local mvec_dis = mvector3.distance
Hooks:PostHook(HUDManager,"_setup_player_info_hud_pd2","spedometer_create_hud",function(self)
	local hm = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2)
	local parent_panel = hm and hm.panel
	if alive(parent_panel) then 
		self._spedometer_box = HUDBGBox_create(parent_panel, {
			name = "spedometer_box",
			w = BOX_W,
			h = BOX_H,
			x = BOX_X,
			y = BOX_Y
		}, {
			blend_mode = "add"
		})
		
		self._spedometer_label = self._spedometer_box:text({
			name = "spedometer_label",
			color = Color(TEXT_COLOR),
			font = tweak_data.hud_players.ammo_font,
			text = SPEED_LABEL,
			vertical = "center",
			align = "left",
			x = TEXT_H_MARGIN,
			layer = 2,
			font_size = FONT_SIZE
		})
		self._spedometer_text = self._spedometer_box:text({
			name = "spedometer_counter",
			color = Color(TEXT_COLOR),
			font = tweak_data.hud_players.ammo_font,
			text = "SPEED: ",
			vertical = "center",
			x = -TEXT_H_MARGIN,
			align = "right",
			layer = 2,
			font_size = FONT_SIZE
		})
		self._spedometer_last_pos = Vector3()
		self._spedometer_last_t = 0
		self:add_updator("spedometer_update",function(t,dt)
			local player = managers.player and managers.player:local_player()
			if player and alive(player) then 
				if self._spedometer_last_t + AVERAGE_INTERVAL_TIME < t then 
					if dt > 0 then 
						local current_pos = player:movement():m_pos()
						local distance = mvec_dis(current_pos,self._spedometer_last_pos)
						local speed = distance / (t - self._spedometer_last_t)
						
						if ROUND_TO_NEAREST > 0 then 
							speed = math.round(speed / ROUND_TO_NEAREST) * ROUND_TO_NEAREST
						end
						
						self._spedometer_text:set_text(string.format(FORMAT_STRING,speed))
						
						self._spedometer_last_pos = mvec_copy(current_pos)
						self._spedometer_last_t = t
					end
				end
			else
				self._spedometer_text:set_text("NO PLAYER")
			end
		end)
		
			
	else
		log("ERROR: parent panel not found")
	end
end)